﻿package galileu
{
	import galileu.SliderLabel;
	
	import fl.data.DataProvider;
	import fl.events.SliderEvent;
	import fl.transitions.Tween;
	import fl.transitions.TweenEvent;
	import fl.transitions.easing.Regular;
	
	import flash.display.MovieClip;
	import flash.display.SimpleButton;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Point;
	import flash.media.Sound;
	import flash.net.URLRequest;
	
	public class Main extends MovieClip
	{
		private static const TIME_SECONDS:int = 1;
		private static const g:Number = 9.8;
		private static const NUMCASAS:Number = 6;
		
		private var nTime:int; //tempo em segundo
		private var nRow:int;
		private var arrMatrix:Array;
		private var dpData:DataProvider;
		private var sllHeight:SliderLabel;
		private var sllAngle:SliderLabel;
		private var cBall:Circle;
		private var tween:Tween;
		private var gravI:Number;
		private var gravX:Number;
		private var gravY:Number;
		private var nDeltaT:Number;
		private var nRadianAngle:Number;
		private var sndRowSound:Sound;
		
		public function Main()
		{
			removeChild(btnReinit);
			
			sndRowSound = new Sound();
			sndRowSound.load(new URLRequest("sons/linhaDaTabela.mp3"));
			
			dpData = new DataProvider();
			arrMatrix = [];
			
			cBall = new Circle();
			cBall.setColor(0x000000);
			cBall.setRadius(6);
			cBall.drawMe();
			rmpRamp.setBall(cBall);
			
			slHeight.minimum = 100;
			slHeight.maximum = 170;
			slHeight.value = 170;
			slHeight.addEventListener(SliderEvent.CHANGE, updateRamp);
			slHeight.addEventListener(SliderEvent.THUMB_DRAG, updateRamp);
			sllHeight = new SliderLabel(slHeight, lblHeight);
			
			slAngle.minimum = 15;
			slAngle.maximum = 60;
			slAngle.value = 15;
			slAngle.addEventListener(SliderEvent.CHANGE, updateRamp);
			slAngle.addEventListener(SliderEvent.THUMB_DRAG, updateRamp);
			sllAngle = new SliderLabel(slAngle, lblAngle);
			
			updateRamp();
			
			table.setSize(400, 175);
			table.columns = ["t", "s(t)", "∆s", "∆²s", "∆³s"];
			table.sortableColumns = false;
			table.dataProvider = dpData;
			comecar.addEventListener(MouseEvent.CLICK, forward);
		}
		
		private function updateRamp(_myEvent:Event = null):void
		{
			rmpRamp.build(slHeight.value / Math.tan(MyMath.convertDegreeToRadian(slAngle.value)), slHeight.value);
		}
		
		private function reinit():void
		{
			if (tween != null)
				tween.stop();
			tween = null;
			nTime = 0;
			rmpRamp.reinit();
			dpData = new DataProvider();
			table.dataProvider = dpData;
			nRow = 0;
		}
		
		private function arredondamento(valor:Number,casas:uint):Number 
		{
			var novo=Math.round(valor * Math.pow(10, casas)) / Math.pow(10, casas);
			return novo;
		}
		
		private function calculateTable():void
		{
			var i:int;
			var tempo:Number = 0;
			var tempoMax = Math.sqrt(rmpRamp.getWidth() / (0.5 * gravX));
			arrMatrix = [];
			while (tempo < tempoMax) 
			{
				arrMatrix.push([tempo,arredondamento(0.5 * gravI * (Math.pow(tempo, 2)), NUMCASAS), "", "", ""]);
				tempo++;
			}
			arrMatrix.push([tempo,arredondamento(0.5*gravI*(Math.pow(tempo,2)),NUMCASAS),"","",""]);
			for (i=0; i<tempo; i++) 
			{
				arrMatrix[i][2]=arredondamento(arrMatrix[i+1][1]-arrMatrix[i][1],NUMCASAS-1);
			}
			for (i=0; i<tempo-1; i++) 
			{
				arrMatrix[i][3]=arredondamento(arrMatrix[i+1][2]-arrMatrix[i][2],NUMCASAS-4);
			}
			for (i=0; i<tempo-2; i++) 
			{
				arrMatrix[i][4]=arredondamento(arrMatrix[i+1][3]-arrMatrix[i][3],NUMCASAS);
			}
		}
		
		private function calculate():void
		{
			nRadianAngle = MyMath.convertDegreeToRadian(slAngle.value);
			gravI = g * Math.sin(nRadianAngle);
			gravX = gravI * Math.cos(nRadianAngle);
			gravY = gravI * Math.sin(nRadianAngle);
			calculateTable();
		}
		
		private function updateTween(_evt:TweenEvent):void
		{
			var nCurrentTime:Number = nTime + _evt.target.time / _evt.target.duration * nDeltaT;
			var t2:Number = Math.pow(nCurrentTime, 2);
			var posicaoAtual:Point = new Point((0.5 * (gravX) * t2), (0.5 * (gravY) * t2));
			if (posicaoAtual.y < rmpRamp.getHeight()) 
			{
				rmpRamp.getBall().x = posicaoAtual.x;
				rmpRamp.getBall().y = posicaoAtual.y - rmpRamp.getBall().getRadius();
			}
		}
		
		private function getTime(_xDistance:Number):Number
		{
			return Math.sqrt(2 * _xDistance / gravX);
		}
		
		private function updateScreen():void
		{
			var nNewTime:Number;
			var nXDistance:Number = Math.min(rmpRamp.getWidth(),
											arrMatrix[nRow][1] * Math.cos(nRadianAngle));
			nNewTime = (nXDistance == rmpRamp.getWidth()) ? getTime(nXDistance) : arrMatrix[nRow][0];
			nDeltaT = nNewTime - nTime;
			tween = new Tween(	rmpRamp.getBall(), 
								"x",
								Regular.easeInOut,
								rmpRamp.getBall().x,
								nXDistance,
								nDeltaT / TIME_SECONDS,
								true);
		}
		
		private function createTween():void
		{
			updateScreen();
			tween.addEventListener(TweenEvent.MOTION_CHANGE, updateTween);
			tween.addEventListener(TweenEvent.MOTION_FINISH, atualizaFrame);
		}
		
		private function atualizaFrame(_myEvent:TweenEvent = null):void 
		{
			if(nRow < arrMatrix.length)
			{
				dpData.addItem({"t" : arrMatrix[nRow][0],
								"s(t)" : arrMatrix[nRow][1],
								"∆s" : arrMatrix[nRow][2],
								"∆²s" : arrMatrix[nRow][3],
								"∆³s" : arrMatrix[nRow][4]});
				nTime = arrMatrix[nRow][0];
				sndRowSound.play();
				nRow++;
				if (nRow < arrMatrix.length)
				{
					createTween();
				}
				else
					tween = null;
			}
		}
		
		private function forward(event:MouseEvent):void 
		{
			reinit();
			if (contains(comecar))
			{
				comecar.removeEventListener(MouseEvent.CLICK, forward);
				btnReinit.addEventListener(MouseEvent.CLICK, forward);
				removeChild(comecar);
				addChild(btnReinit);
				calculate();
				atualizaFrame();
			}
			else
			{
				DrawingUtil.removeMe(btnContinue);
				DrawingUtil.addMe(btnPause, this);
				removeChild(btnReinit);
				addChild(comecar);
				comecar.addEventListener(MouseEvent.CLICK, forward);
				btnReinit.removeEventListener(MouseEvent.CLICK, forward);
			}
		}
	}
}