﻿package galileu
{
	import flash.geom.Point;
	
	public class MyMath
	{
		public static const ERROR_TO_NUMBER:Number = .1;
		public static const ERROR_TO_ANGLE:Number = 3;
		
		// 1 até 3
		public static function getRomanNumber(_number:int):String
		{
			switch(_number)
			{
				case 1:
					return "I";
				case 2:
					return "II";
				case 3:
					return "III";
			}
			
			return "";
		}
		
		public static function getPositiveRemainder(_dividend:int, _divisor:int):int
		{
			var remainder:int = _dividend % _divisor;
			if (remainder < 0)
				remainder = _divisor + remainder;
				
			return remainder;
		}
		
		public static function isInClosedInterval(number:Number, initialNumber:Number, finalNumber:Number):Boolean
		{
			if (number >= initialNumber && number <= finalNumber)
				return true;
				
			return false;
		}
		
		public static function convertAngleToNumber(_strAngle:String):Number
		{
			return Number(_strAngle.slice(0, _strAngle.length - 1));
		}
		
		public static function getAngle(edge1:Array, edge2:Array, error:Number=ERROR_TO_ANGLE):Number {
			if (Comparator.equalPointsWithError(edge1[0], edge2[1], error) &&
						Comparator.equalPointsWithError(edge1[1], edge2[0], error))
			{
				return 0;
			}
			if (Comparator.equalPointsWithError(edge1[0], edge2[1], error)) {
				return calculateAngle(edge2[0],edge2[1],edge1[1].subtract(subtractPoint(edge1[0], edge2[1])));
			} else if (Comparator.equalPointsWithError(edge1[1], edge2[0], error)) {
				return calculateAngle(edge1[0],edge1[1],edge2[1].subtract(subtractPoint(edge2[0], edge1[1])));
			}
			return Number.NaN;
		}
		
		public static function getLengthEdge(edge:Array):Number
		{
			var firstPoint:Point=edge[0];
			var lastPoint:Point=edge[1];
			var difXPow2:Number=Math.pow((lastPoint.x-firstPoint.x),2);
			var difYPow2:Number=Math.pow((lastPoint.y-firstPoint.y),2);
			var distancia:Number=Math.sqrt(difXPow2+difYPow2);
			return distancia;
		}
		
		public static function equalAnglesWithError(number1:Number, number2:Number):Boolean {
			return (Math.abs(number1 - number2) < .2);
		}
		
		private static function subtractPoint(v1:Point, v2:Point):Point
		{
			return new Point(v1.x - v2.x, v2.y - v1.y);
		}
		
		public static function convertRadianToDegree(_angle:Number):Number {
			return _angle / Math.PI * 180 % 360;
		}
		
		public static function convertDegreeToRadian(_angle:Number):Number {
			return (_angle * Math.PI / 180) % (2 * Math.PI);
		}
		
		private static function calculateAngleByVectors(a:Point, b:Point):Number
		{
			var num1:Number = (a.x) * (b.x) + (a.y) * (b.y);
			var center:Point = new Point(0, 0);
			var num2:Number = Point.distance(a, center) * Point.distance(b, center);
			
			var div:Number = num1 / num2;
			var acos:Number = convertRadianToDegree(Math.acos(div));
			
			return acos;
		}
		// 0 <= angulo < 360
		private static function calculateAngle(a:Point, b:Point/* centro */, c:Point):Number {
			var iAxis:Point = new Point(1, 0);
			var jAxis:Point = new Point(0, 1);
			
			var firstVector:Point = new Point(b.x - a.x, a.y - b.y); // vetor que entra
			var secondVector:Point = new Point(c.x - b.x, b.y - c.y); // vetor que sai
			
			var iAngle:Number;
			var jAngle:Number;
			
			var firstAngle:Number;
			var secondAngle:Number;
			
			// cálculo do ângulo do primeiro vetor com relação ao eixo i
			iAngle = calculateAngleByVectors(iAxis, firstVector); 
			jAngle = calculateAngleByVectors(jAxis, firstVector); 
			
			if (iAngle == 0)
				firstAngle = 180;
			else if (iAngle == 180)
				firstAngle = 0;
			else if (jAngle >= 90 && jAngle <= 180) // como é o vetor que entra, já tem pelo menos 180º de ângulo
			{
				firstAngle = 180 + iAngle;
			}
			else
			{
				firstAngle = 180 - iAngle;
			}
			
			// cálculo do ângulo do segundo vetor com relação ao eixo i
			iAngle = calculateAngleByVectors(iAxis, secondVector);
			jAngle = calculateAngleByVectors(jAxis, secondVector);
			if (iAngle == 0 || iAngle == 180)
				secondAngle = iAngle;
			else if (jAngle >= 90 && jAngle <= 180) 
			{
				secondAngle = iAngle;
			}
			else // como é o vetor que sai, já tem pelo menos 180º de ângulo
			{
				secondAngle = 360 - iAngle;
			}
			
			if (equalAnglesWithError(firstAngle, secondAngle))
				return 0;
				
			if (firstAngle < secondAngle)
				return secondAngle - firstAngle;
				
			return (360 - firstAngle) + secondAngle;
			
		}
	}
}