﻿package fita
{
	import fita.SliderLabel;
	
	import fl.data.DataProvider;
	import fl.transitions.Tween;
	import fl.transitions.TweenEvent;
	import fl.transitions.easing.Regular;
	
	import flash.display.MovieClip;
	import flash.display.SimpleButton;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.media.Sound;
	import flash.net.URLRequest;
	import flash.text.TextField;
	
	public class Main extends MovieClip
	{
		private static const MAX_TURNS:int = 1000;
		private static const PIXEL_TURNS:int = 25; // quantas voltas correspondem a um pixel
		private static const TIME_SECONDS:int = 270;
		private static const TURN_WIDTH:int = 2;
		private static const INICIO:int = 0;
		
		private var deltaVoltas:SliderLabel;
		private var N:int; //num de voltas
		private var T:int; //tempo em segundo
		private var linha:int;
		private var dp:DataProvider;
		private var matriz:Array;
		private var tween:Tween;
		private var currentRightTurn:int;
		private var currentLeftTurn:Number;
		private var leftTurnDelta:Number;
		private var currentRightTurnTime:Number;
		private var initialLeftWidth:Number;
		private var nTime:Number;
		private var deltaNSound:Sound;
		private var currentDeltaN:int;
			
		public function Main()
		{
			removeChild(btnReinit);
			removeChild(btnContinue);
			
			deltaNSound = new Sound();
			deltaNSound.load(new URLRequest("sons/deltaN.mp3"));
			
			leftSide.drawMe();
			rightSide.setOuterRadius(rightSide.getInnerRadius());
			rightSide.drawMe();
			
			deltaN.minimum = 50;
			deltaN.maximum = 400;
			deltaN.snapInterval = 50;
			deltaN.value = 100;
			deltaVoltas = new SliderLabel(deltaN,deltaNLabel);
			N = INICIO; //num de voltas
			T = 0; //tempo em segundo
			linha = 0;
			dp = new DataProvider();
			matriz = [];
			
			grid.setSize(290, 285);
			grid.columns = ["N","t","∆t","∆²t"];
			grid.dataProvider = dp;
			grid.sortableColumns = false;
			comecar.addEventListener(MouseEvent.CLICK, adiante);
			btnPause.addEventListener(MouseEvent.CLICK, doPause);
			btnContinue.addEventListener(MouseEvent.CLICK, doContinuation);
		}
		
		private function doPause(_myEvent:MouseEvent):void
		{
			if (tween != null)
			{
				removeChild(btnPause);
				addChild(btnContinue);
				tween.stop();
			}
		}
		
		private function doContinuation(_myEvent:MouseEvent):void
		{
			removeChild(btnContinue);
			addChild(btnPause);
			tween.resume();
		}
		
		private function updateTween(_evt:TweenEvent):void
		{
			var nProgress:Number = _evt.target.time / _evt.target.duration;
			var nTurns:Number = nProgress * leftTurnDelta;
			tfTurns.text = "Voltas: " + Math.floor(currentRightTurn - (1 - nProgress) * PIXEL_TURNS);
			tfTime.text = "Tempo: " + Math.floor(nTime - (1 - nProgress) * currentRightTurnTime);
			rightSide.updateLine();
			_evt.target.obj.height = _evt.target.obj.width;
			leftSide.getOuterCircle().width = initialLeftWidth - 2 * nTurns / PIXEL_TURNS;
			leftSide.getOuterCircle().height = leftSide.getOuterCircle().width;
			leftSide.updateLine();
		}
		
		private function continueTweening(_evt:TweenEvent):void
		{
			atualizaFrame();
		}
		
		private function getTurnTime():Number
		{
			return getTime(currentRightTurn + PIXEL_TURNS) - getTime(currentRightTurn);
		}
		
		private function getTurnWidth():Number
		{
			return currentDeltaN / deltaN.minimum * TURN_WIDTH;
		}
		
		private function adiante(event:MouseEvent):void 
		{
			reinit();
			if (contains(comecar))
			{
				currentDeltaN = deltaN.value;
				comecar.removeEventListener(MouseEvent.CLICK, adiante);
				btnReinit.addEventListener(MouseEvent.CLICK, adiante);
				removeChild(comecar);
				addChild(btnReinit);
				calculos();
				atualizaFrame();
			}
			else
			{
				DrawingUtil.removeMe(btnContinue);
				DrawingUtil.addMe(btnPause, this);
				removeChild(btnReinit);
				addChild(comecar);
				comecar.addEventListener(MouseEvent.CLICK, adiante);
				btnReinit.removeEventListener(MouseEvent.CLICK, adiante);
			}
		}
		
		private function reinitLabel(_label:TextField):void
		{
			_label.text = _label.text.split(" ")[0] + " 0";
		}
		
		private function reinitLabels():void
		{
			reinitLabel(tfTurns);
			reinitLabel(tfTime);
		}
		
		private function reinit():void
		{
			if (tween != null)
				tween.stop();
			tween = null;
			nTime = 0;
			currentRightTurn = 0;
			currentLeftTurn = 0;
			leftSide.reinit();
			rightSide.reinit(rightSide.getInnerRadius());
			reinitLabels();
			dp = new DataProvider();
			grid.dataProvider = dp;
			linha = 0;
			N = INICIO;
		}
		
		private function updateScreen():void
		{
			currentRightTurnTime = getTurnTime();
			initialLeftWidth = leftSide.getOuterCircle().width;
			leftTurnDelta = getLeftTurns(getLeftTime(currentLeftTurn) - currentRightTurnTime) -
							currentLeftTurn;
			tween = new Tween(	rightSide.getOuterCircle(), 
								"width",
								Regular.easeInOut,
								rightSide.getOuterCircle().width,
								rightSide.getOuterCircle().width + 2,
								currentRightTurnTime / TIME_SECONDS,
								true);
			currentRightTurn += PIXEL_TURNS;
			currentLeftTurn = getLeftTurns(7200 - getTime(currentRightTurn + PIXEL_TURNS));
		}
		
		private function initScreen():void
		{
		}
		
		private function createTween():void
		{
			updateScreen();
			tween.addEventListener(TweenEvent.MOTION_CHANGE, updateTween);
			tween.addEventListener(TweenEvent.MOTION_FINISH, continueTweening);
		}
		
		private function getLeftTurns(_time:Number):Number
		{
			return -250 * Math.pow(0.008 * _time + 27.04, 0.5) + 2300;
		}
		
		private function getLeftTime(_turn:int):Number
		{
			return 0.002 * Math.pow(_turn, 2) - 9.2 * (_turn) + 7200;
		}
		
		private function getTime(_turn:int):Number
		{
			return 0.002 * Math.pow(_turn,2) + 5.2 * _turn;
		}
		
		private function calculos():void
		{
			var i:int;
			
			matriz = [];
			while (N <= MAX_TURNS) 
			{
				T = getTime(N);
				matriz.push([N, arredondamento(T, 2), "", ""]);
				N += deltaN.value;
			}
			for(i = 1; i < matriz.length; i++)
			{
				matriz[i - 1][2] = matriz[i][1] - matriz[i - 1][1];
			}
			for(i = 2; i < matriz.length; i++)
			{
				matriz[i - 2][3]=matriz[i - 1][2] - matriz[i - 2][2];
			}
		}
		
		private function atualizaFrame():void 
		{
			if(linha < matriz.length)
			{
				if (currentRightTurn % currentDeltaN == 0)
				{
					dp.addItem({"N" : matriz[linha][0],
								"t" : matriz[linha][1],
								"∆t" : matriz[linha][2],
								"∆²t" : matriz[linha][3]});
					deltaNSound.play();
					linha++;
				}
				if (linha < matriz.length)
				{
					createTween();
					nTime += currentRightTurnTime;
				}
				else
					tween = null;
			}
		}
		
		private function arredondamento(valor:Number, casas:uint):Number 
		{
			var novo = Math.round(valor * Math.pow(10, casas)) / Math.pow(10, casas);
			return novo;
		}
	}
}